//+------------------------------------------------------------------+
//|                                             ROC MTF.mq4
//|                                    Copyright  2006, Robert Hill   
//                                  modified by Boxter
//+------------------------------------------------------------------+

#property  copyright "Copyright  2006, Robert Hill"
//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 1
#property  indicator_color1  Red
//---- indicator parameters
extern string TimeFrame = "Current time frame";
extern int RPeriod = 5;
extern bool UsePercent = true;
extern int  PriceSmoothing      =   6;
extern int  PriceSmoothingPhase =   0;
extern bool   Interpolate = true;

//---- indicator buffers
double RateOfChange[];

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init(){
//---- drawing settings
   SetIndexStyle(0, DRAW_LINE);
   SetIndexDrawBegin(0, RPeriod);
   IndicatorDigits(Digits + 1);
//---- indicator buffers mapping
   if(!SetIndexBuffer(0, RateOfChange))
       Print("cannot set indicator buffers!");
       
   indicatorFileName   = WindowExpertName();
   returnBars          = TimeFrame=="returnBars";          if (returnBars)          return(0);
   calculateValue      = TimeFrame=="calculateValue";      if (calculateValue)      return(0);
   timeFrame = stringToTimeFrame(TimeFrame);
          
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName(timeFrameToString(timeFrame)+ " ROC2 MTF(" + RPeriod + ")");
//---- initialization done
   return(0);
  }
  
//+------------------------------------------------------------------+
//| ROC calculation                          |
//+------------------------------------------------------------------+
int start(){
   int limit,r,i;
   double ROC, CurrentClose, PrevClose;
   int counted_bars = IndicatorCounted();
//---- check for possible errors
   if(counted_bars < 0) 
       return(-1);
//---- last counted bar will be recounted
   if(counted_bars > 0) 
       counted_bars--;
   limit = MathMin(Bars - counted_bars,Bars-1);
   
   if (returnBars) { RateOfChange[0] = limit+1; return(0); }

//Current TF calculation   
   if (calculateValue || timeFrame==Period()){
      for(i=limit; i>=0; i--){
         CurrentClose = iClose(NULL, 0, i);
         PrevClose = iClose(NULL, 0, i + RPeriod);
         ROC = CurrentClose - PrevClose;
         if(UsePercent){
            if(PrevClose != 0) RateOfChange[i] = 100 * ROC / PrevClose;
         }
         else RateOfChange[i] = ROC;
         RateOfChange[i] = iSmooth(RateOfChange[i],PriceSmoothing,PriceSmoothingPhase,i,0);
      }   
      return(0);
   }
  
//mtf Calculation
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(r=limit; r>=0; r--){
      int y = iBarShift(NULL,timeFrame,Time[r]);               
      RateOfChange[r]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RPeriod,UsePercent,PriceSmoothing,PriceSmoothingPhase,0,y);

      if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[r-1])) continue;
      datetime time = iTime(NULL,timeFrame,y);
      for(int n = 1; r+n < Bars && Time[r+n] >= time; n++) continue;	    
      for(int k = 1; k < n; k++) {RateOfChange[r+k] = RateOfChange[r] + (RateOfChange[r+n] - RateOfChange[r]) * k/n;}
   }
   return(0);
}
//+------------------------------------------------------------------
//|     Smooth function                                                            
//+------------------------------------------------------------------
double wrk[][50];

#define bsmax  5
#define bsmin  6
#define volty  7
#define vsum   8
#define avolty 9

double iSmooth(double price, double length, double phase, int i, int s=0){
   if (ArrayRange(wrk,0) != Bars) ArrayResize(wrk,Bars);

   if (length<=1) return(price);
   int r = Bars-i-1; 
   if (r==0) { for(int k=0; k<7; k++) wrk[0][k+s]= price; for(; k<10; k++) wrk[0][k+s]= 0; return(price); }
   
   double len1 = MathMax(MathLog(MathSqrt(0.5*(length-1)))/MathLog(2.0)+2.0,0);
   double pow1 = MathMax(len1-2.0,0.5);
   double del1 = price - wrk[r-1][bsmax+s];
   double del2 = price - wrk[r-1][bsmin+s];
	
   wrk[r][volty+s] = 0;
   if(MathAbs(del1) > MathAbs(del2)) wrk[r][volty+s] = MathAbs(del1); 
   if(MathAbs(del1) < MathAbs(del2)) wrk[r][volty+s] = MathAbs(del2); 
   wrk[r][vsum+s] =	wrk[r-1][vsum+s] + 0.1*(wrk[r][volty+s]-wrk[r-10][volty+s]);
      
   double avgLen = MathMin(MathMax(4.0*length,30),150);
   if (r<avgLen){
      double avg = wrk[r][vsum+s];  
      for (k=1; k<avgLen && (r-k)>=0 ; k++) avg += wrk[r-k][vsum+s];
      avg /= k;
   }
   else avg = (wrk[r-1][avolty+s]*avgLen-wrk[r-toInt(avgLen)][vsum+s]+wrk[r][vsum+s])/avgLen;
                                                          
   wrk[r][avolty+s] = avg;                                           
   if (wrk[r][avolty+s] > 0)double dVolty = wrk[r][volty+s]/wrk[r][avolty+s]; 
   else dVolty = 0;   
	if (dVolty > MathPow(len1,1.0/pow1)) dVolty = MathPow(len1,1.0/pow1);
   if (dVolty < 1)                      dVolty = 1.0;
	        
   double pow2 = MathPow(dVolty, pow1);
   double len2 = MathSqrt(0.5*(length-1))*len1;
   double Kv   = MathPow(len2/(len2+1), MathSqrt(pow2));		
	
   if (del1 > 0) wrk[r][bsmax+s] = price; else wrk[r][bsmax+s] = price - Kv*del1;
   if (del2 < 0) wrk[r][bsmin+s] = price; else wrk[r][bsmin+s] = price - Kv*del2;

   double R     = MathMax(MathMin(phase,100),-100)/100.0 + 1.5;
   double beta  = 0.45*(length-1)/(0.45*(length-1)+2);
   double alpha = MathPow(beta,pow2);

   wrk[r][0+s] = price + alpha*(wrk[r-1][0+s]-price);
   wrk[r][1+s] = (price - wrk[r][0+s])*(1-beta) + beta*wrk[r-1][1+s];
   wrk[r][2+s] = (wrk[r][0+s] + R*wrk[r][1+s]);
   wrk[r][3+s] = (wrk[r][2+s] - wrk[r-1][4+s])*MathPow((1-alpha),2) + MathPow(alpha,2)*wrk[r-1][3+s];
   wrk[r][4+s] = (wrk[r-1][4+s] + wrk[r][3+s]); 

   return(wrk[r][4+s]);
}

int toInt(double value) { return(value); }   

//+------------------------------------------------------------------+
//|  mtf Functions
//+------------------------------------------------------------------+
string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

int stringToTimeFrame(string tfs){
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
      if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
   return(Period());
}

string timeFrameToString(int tf){
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
      if (tf==iTfTable[i]) return(sTfTable[i]);
   return("");
}

string StringUpperCase(string str){
   string   s = str;
   for (int length=StringLen(str)-1; length>=0; length--){
      int tchar = StringGetChar(s, length);
      if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
         s = StringSetChar(s, length, tchar - 32);
      else if(tchar > -33 && tchar < 0)
         s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}